describe Celsius::Primo::Adapter do
  let(:adapter_options) do
    {
      institution: "PAD",
      languages: [
        "ger",
        "eng"
      ],
      locations:[
        {
          type: "local",
          value: "scope:(PAD_ALEPH)"
        }
      ],
      soap_api_options: {
        searcher_url: "http://primo.kobv.de/primo_library/libweb/webservices/services/searcher"
      }
    }
  end

  let(:adapter) do
    described_class.new(adapter_options)
  end

  describe "#mget" do
    it "returns a normalzed mget result", :vcr do
      mget_request = YAML.load(read_asset("adapter/mget/mget_request.yml"))
      expected_mget_result = YAML.load(read_asset("adapter/mget/mget_result.yml"))

      expect(adapter.mget(mget_request)).to eq(expected_mget_result)
    end
  end
  
  describe "#search" do
    asset_dir = "adapter/search"

    context "if given search request is not a hash" do
      it "raises an error" do
        expect { adapter.search(:no_array) }.to raise_error(ArgumentError, /hash/)
      end
    end


    context "if languages is neither set on the adapter nor given" do
      let(:adapter) do
        described_class.new(adapter_options.tap { |options| options.delete(:languages) })
      end

      it "raises an error" do
        expect { adapter.search({}) }.to raise_error(ArgumentError, /languages/)
      end
    end

    context "if locations is neither set on the adapter nor given" do
      let(:adapter) do
        described_class.new(adapter_options.tap { |options| options.delete(:locations) })
      end

      it "raises an error" do
        expect { adapter.search({}) }.to raise_error(ArgumentError, /locations/)
      end
    end

    context "if institution is neither set on the adapter nor given" do
      let(:adapter) do
        described_class.new(adapter_options.tap { |options| options.delete(:institution) })
      end

      it "raises an error" do
        expect { adapter.search({}) }.to raise_error(ArgumentError, /institution/)
      end
    end

    #
    # Create specs dynamically from the given scenarios
    #
    [
      "#{asset_dir}/scenarios/complex_request",
      "#{asset_dir}/scenarios/empty_result",
      "#{asset_dir}/scenarios/missing_facets",
      "#{asset_dir}/scenarios/result_with_multiple_publishers",
      "#{asset_dir}/scenarios/simple_request"
    ].each do |directory|
      it "passes the scenario \"#{directory.split("/").last}\"", :vcr do
        search_request = YAML.load(read_asset("#{directory}/search_request.yml"))
        expected_search_result = YAML.load(read_asset("#{directory}/search_result.yml"))

        expect(search_result = adapter.search(search_request)).to eq(expected_search_result)
      end
    end

    context "if a the adapter specifies a timeout" do
      context "if the server does not respond within this timeout" do
        let(:timeout) { 0.1 } # decimals can be used the specifiy milliseconds

        it "raises an Timeout::Error" do
          require "timeout"
          allow(HTTPI).to receive(:post) { sleep(timeout * 2) }
          search_request = YAML.load(read_asset("adapter/search/scenarios/simple_request/search_request.yml"))

          expect { adapter.search(search_request, timeout: timeout) }.to raise_error(Timeout::Error)
        end
      end
    end
  end
end
